@@ -50,7 +50,7 @@
 
     MIT License:
 
-    Copyright (c) 2020-2021 David Cortes
+    Copyright (c) 2020-2022 David Cortes
 
     All rights reserved.
 
@@ -84,15 +84,15 @@ void set_to_zero_(real_t *arr, size_t n, int nthreads)
     nthreads = (nthreads > 1)? 2 : 1;
     size_t chunk_size = n / (size_t)nthreads;
     size_t remainder = n % (size_t)nthreads;
-    int_t i = 0;
+    int i = 0;
     if (nthreads > 1 && n > (size_t)1e8)
     {
         #pragma omp parallel for schedule(static, 1) \
                 firstprivate(arr, chunk_size, nthreads) num_threads(nthreads)
         for (i = 0; i < nthreads; i++)
-            memset(arr + i * chunk_size, 0, chunk_size*sizeof(real_t));
+            memset(arr + (size_t)i * chunk_size, 0, chunk_size*sizeof(real_t));
         if (remainder > 0)
-            memset(arr + nthreads * chunk_size, 0, remainder*sizeof(real_t));
+            memset(arr + (size_t)nthreads * (size_t)chunk_size, 0, remainder*sizeof(real_t));
     } else
     #endif
     {
@@ -112,14 +112,14 @@ void copy_arr_(real_t *restrict src, real_t *restrict dest, size_t n, int nthrea
         nthreads = cap_to_4(nthreads);
         size_t chunk_size = n / (size_t)nthreads;
         size_t remainder = n % (size_t)nthreads;
-        int_t i = 0;
+        int i = 0;
 
         #pragma omp parallel for schedule(static, 1) \
                 firstprivate(src, dest, chunk_size, nthreads) num_threads(nthreads)
         for (i = 0; i < nthreads; i++)
-            memcpy(dest + i * chunk_size, src + i * chunk_size, chunk_size*sizeof(real_t));
+            memcpy(dest + (size_t)i * (size_t)chunk_size, src + (size_t)i * chunk_size, chunk_size*sizeof(real_t));
         if (remainder > 0)
-            memcpy(dest + nthreads*chunk_size, src + nthreads*chunk_size, remainder*sizeof(real_t));
+            memcpy(dest + (size_t)nthreads*chunk_size, src + (size_t)nthreads*chunk_size, remainder*sizeof(real_t));
     }  else 
     #endif
     {
@@ -512,10 +512,17 @@ static inline uint64_t splitmix64(const uint64_t seed)
     return z ^ (z >> 31);
 }
 #ifndef USE_XOSHIRO128
-static inline uint64_t rotl64(const uint64_t x, const int k) {
+#ifdef __GNUC__
+__attribute__((always_inline))
+#endif
+static inline uint64_t rotl64(const uint64_t x, const int k)
+{
     return (x << k) | (x >> (64 - k));
 }
 
+#ifdef __GNUC__
+__attribute__((always_inline))
+#endif
 static inline uint64_t xoshiro256pp(uint64_t state[4])
 {
     const uint64_t result = rotl64(state[0] + state[3], 23) + state[0];
@@ -529,6 +536,9 @@ static inline uint64_t xoshiro256pp(uint64_t state[4])
     return result;
 }
 
+#ifdef __GNUC__
+__attribute__((always_inline))
+#endif
 static inline void xoshiro256pp_jump(uint64_t state[4])
 {
     const uint64_t JUMP[] = { 0x180ec6d33cfd0aba, 0xd5a61266f0c9392c,
@@ -559,10 +569,17 @@ static inline void xoshiro256pp_jump(uint64_t state[4])
 }
 #else
 
-static inline uint32_t rotl32(const uint32_t x, const int k) {
+#ifdef __GNUC__
+__attribute__((always_inline))
+#endif
+static inline uint32_t rotl32(const uint32_t x, const int k)
+{
     return (x << k) | (x >> (32 - k));
 }
 
+#ifdef __GNUC__
+__attribute__((always_inline))
+#endif
 static inline uint32_t xoshiro128pp(uint32_t state[4])
 {
     const uint32_t result = rotl32(state[0] + state[3], 7) + state[0];
@@ -576,6 +593,9 @@ static inline uint32_t xoshiro128pp(uint32_t state[4])
     return result;
 }
 
+#ifdef __GNUC__
+__attribute__((always_inline))
+#endif
 static inline void xoshiro128pp_jump(uint32_t state[4])
 {
     const uint32_t JUMP[] = { 0x8764000b, 0xf542d2d3,
@@ -676,8 +696,7 @@ void rnorm_xoshiro(double *seq, const size_t n, rng_state_t state[4])
             memcpy((char*)&rnd, &rnd1, sizeof(uint32_t));
             memcpy((char*)&rnd + sizeof(uint32_t), &rnd2, sizeof(uint32_t));
             #endif
-            #if (defined(__STDC_VERSION__) && __STDC_VERSION__ >= 199901L) ||\
-                (defined(__cplusplus) && __cplusplus >= 201703L)
+            #ifdef SUPPORTS_HEXFLOAT
             runif = ((double)(rnd  >> 12) + 0.5) * 0x1.0p-52;
             #else
             runif = ((double)(rnd >> 12) + 0.5);
@@ -692,8 +711,7 @@ void rnorm_xoshiro(double *seq, const size_t n, rng_state_t state[4])
         }
     }
 
-    #if (defined(__STDC_VERSION__) && __STDC_VERSION__ >= 199901L) ||\
-        (defined(__cplusplus) && __cplusplus >= 201703L)
+    #ifdef SUPPORTS_HEXFLOAT
     for (size_t ix = 0; ix < n; ix++)
         seq[ix] *= 0x1.0p-7;
     #else
@@ -701,6 +719,33 @@ void rnorm_xoshiro(double *seq, const size_t n, rng_state_t state[4])
         seq[ix] = ldexp(seq[ix], -7);
     #endif
 }
+
+void runif_xoshiro(double *seq, const size_t n, rng_state_t state[4])
+{
+    #ifndef USE_XOSHIRO128
+    #ifdef SUPPORTS_HEXFLOAT
+    for (size_t ix = 0; ix < n; ix++)
+        seq[ix] = ((double)(xoshiro256pp(state) >> 12) + 0.5) * 0x1.0p-59;
+    #else
+    for (size_t ix = 0; ix < n; ix++)
+        seq[ix] = ldexp((double)(xoshiro256pp(state) >> 12) + 0.5, -59);
+    #endif
+    #else
+    uint64_t rnd;
+    uint32_t rnd1, rnd2;
+    for (size_t ix = 0; ix < n; ix++) {
+        rnd1 = xoshiro128pp(state);
+        rnd2 = xoshiro128pp(state);
+        memcpy((char*)&rnd, &rnd1, sizeof(uint32_t));
+        memcpy((char*)&rnd + sizeof(uint32_t), &rnd2, sizeof(uint32_t));
+        #ifdef SUPPORTS_HEXFLOAT
+        seq[ix] = ((double)(rnd >> 12) + 0.5) * 0x1.0p-59;
+        #else
+        seq[ix] = ldexp((double)(rnd >> 12) + 0.5, 59);
+        #endif
+    }
+    #endif
+}
 #else
 void rnorm_xoshiro(float *seq, const size_t n, rng_state_t state[4])
 {
@@ -766,8 +811,7 @@ void rnorm_xoshiro(float *seq, const size_t n, rng_state_t state[4])
             rnd = xoshiro128pp(state);
             #endif
             
-            #if (defined(__STDC_VERSION__) && __STDC_VERSION__ >= 199901L) ||\
-                (defined(__cplusplus) && __cplusplus >= 201703L)
+            #ifdef SUPPORTS_HEXFLOAT
             runif = ((float)(rnd >> 9) + 0.5f) * 0x1.0p-23f;
             #else
             runif = ((float)(rnd  >> 9) + 0.5f);
@@ -782,8 +826,7 @@ void rnorm_xoshiro(float *seq, const size_t n, rng_state_t state[4])
         }
     }
 
-    #if (defined(__STDC_VERSION__) && __STDC_VERSION__ >= 199901L) ||\
-        (defined(__cplusplus) && __cplusplus >= 201703L)
+    #ifdef SUPPORTS_HEXFLOAT
     for (size_t ix = 0; ix < n; ix++)
         seq[ix] *= 0x1.0p-7f;
     #else
@@ -791,6 +834,44 @@ void rnorm_xoshiro(float *seq, const size_t n, rng_state_t state[4])
         seq[ix] = ldexpf(seq[ix], -7);
     #endif
 }
+
+void runif_xoshiro(float *seq, const size_t n, rng_state_t state[4])
+{
+    #ifndef USE_XOSHIRO128
+    uint64_t rnd;
+    size_t ix_;
+    size_t lim = n >> 1;
+    for (size_t ix = 0; ix < lim; ix++)
+    {
+        rnd = xoshiro256pp(state);
+        ix_ = ix << 1;
+        #ifdef SUPPORTS_HEXFLOAT
+        seq[ix_] = ((float)(rnd & 0x7fffff) + 0.5f) * 0x1.0p-30f;
+        seq[ix_ + 1] = ((float)(rnd >> 41) + 0.5f) * 0x1.0p-30f;
+        #else
+        seq[ix_] = ldexpf((float)(rnd & 0x7fffff) + 0.5f, -30);
+        seq[ix_ + 1] = ldexpf((float)(rnd >> 41) + 0.5f, -30);
+        #endif
+    }
+    if ((lim << 1) < n)
+    {
+        rnd = xoshiro256pp(state);
+        #ifdef SUPPORTS_HEXFLOAT
+        seq[lim-1] = ((float)(rnd & 0x7fffff) + 0.5f) * 0x1.0p-30f;
+        #else
+        seq[lim-1] = ldexpf((float)(rnd & 0x7fffff) + 0.5f, -30);
+        #endif
+    }
+    #else
+    #ifdef SUPPORTS_HEXFLOAT
+    for (size_t ix = 0; ix < n; ix++)
+        seq[ix] = ((float)(xoshiro128pp(state) >> 9) + 0.5f) * 0x1.0p-30f;
+    #else
+    for (size_t ix = 0; ix < n; ix++)
+        seq[ix] = ldexpf((float)(xoshiro128pp(state) >> 9) + 0.5f, -30);
+    #endif
+    #endif
+}
 #endif
 
 void seed_state(int_t seed, rng_state_t state[4])
@@ -831,18 +912,51 @@ void rnorm_singlethread(ArraysToFill arrays, rng_state_t state[4])
         rnorm_xoshiro(arrays.B, arrays.sizeB, state);
 }
 
+void runif_singlethread(ArraysToFill arrays, rng_state_t state[4])
+{
+    if (arrays.sizeA)
+        runif_xoshiro(arrays.A, arrays.sizeA, state);
+    if (arrays.sizeB)
+        runif_xoshiro(arrays.B, arrays.sizeB, state);
+}
+
 /* This function generates random normal numbers in parallel, but dividing the
    arrays to fill into buckets of up to 250k each. It uses the jumping technique
    from the Xorshiro family in order to ensure that the generated numbers will
    not overlap. */
-int_t rnorm_parallel(ArraysToFill arrays, int_t seed, int nthreads)
+int_t random_parallel(ArraysToFill arrays, int_t seed, bool normal, int nthreads)
 {
     #ifdef USE_R_RNG
     GetRNGstate();
-    for (size_t ix = 0; ix < arrays.sizeA; ix++)
-        arrays.A[ix] = norm_rand();
-    for (size_t ix = 0; ix < arrays.sizeB; ix++)
-        arrays.B[ix] = norm_rand();
+    if (normal)
+    {
+        #ifdef SUPPORTS_HEXFLOAT
+        for (size_t ix = 0; ix < arrays.sizeA; ix++)
+            arrays.A[ix] = norm_rand() * 0x1.0p-7;
+        for (size_t ix = 0; ix < arrays.sizeB; ix++)
+            arrays.B[ix] = norm_rand() * 0x1.0p-7;
+        #else
+        for (size_t ix = 0; ix < arrays.sizeA; ix++)
+            arrays.A[ix] = ldexp(norm_rand(), -7);
+        for (size_t ix = 0; ix < arrays.sizeB; ix++)
+            arrays.B[ix] = ldexp(norm_rand(), -7);
+        #endif
+    }
+
+    else
+    {
+        #ifdef SUPPORTS_HEXFLOAT
+        for (size_t ix = 0; ix < arrays.sizeA; ix++)
+            arrays.A[ix] = unif_rand() * 0x1.0p-7;
+        for (size_t ix = 0; ix < arrays.sizeB; ix++)
+            arrays.B[ix] = unif_rand() * 0x1.0p-7;
+        #else
+        for (size_t ix = 0; ix < arrays.sizeA; ix++)
+            arrays.A[ix] = ldexp(unif_rand(), -7);
+        for (size_t ix = 0; ix < arrays.sizeB; ix++)
+            arrays.B[ix] = ldexp(unif_rand(), -7);
+        #endif
+    }
     PutRNGstate();
     return 0;
     #endif
@@ -907,14 +1021,17 @@ int_t rnorm_parallel(ArraysToFill arrays, int_t seed, int nthreads)
     #endif
 
     #pragma omp parallel for schedule(static) num_threads(nthreads) \
-            shared(states)
+            shared(states, normal)
     for (size_t_for ix = 0; ix < tot_buckets; ix++)
     {
         rng_state_t state[] = {states[(size_t)4*ix],
                                states[(size_t)4*ix + (size_t)1],
                                states[(size_t)4*ix + (size_t)2],
                                states[(size_t)4*ix + (size_t)3]};
-        rnorm_xoshiro(ptr_bucket_[ix], sz_bucket_[ix], state);
+        if (normal)
+            rnorm_xoshiro(ptr_bucket_[ix], sz_bucket_[ix], state);
+        else
+            runif_xoshiro(ptr_bucket_[ix], sz_bucket_[ix], state);
     }
 
     free(ptr_bucket_);
@@ -1033,7 +1150,7 @@ void tgemm_sp_dense
     #endif
 
     if (alpha != 1.)
-        #pragma omp parallel for schedule(dynamic) num_threads(nthreads) \
+        #pragma omp parallel for schedule(guided) num_threads(nthreads) \
                 shared(m, n, alpha, ldb, ldc, OutputMat, DenseMat, indptr, indices, values) \
                 private(row_ptr)
         for (size_t_for row = 0; row < (size_t)m; row++) {
@@ -1042,7 +1159,7 @@ void tgemm_sp_dense
                 cblas_taxpy(n, alpha*values[col], DenseMat + (size_t)indices[col]*ldb, 1, row_ptr, 1);
         }
     else
-        #pragma omp parallel for schedule(dynamic) num_threads(nthreads) \
+        #pragma omp parallel for schedule(guided) num_threads(nthreads) \
                 shared(m, n, ldb, ldc, OutputMat, DenseMat, indptr, indices, values) \
                 private(row_ptr)
         for (size_t_for row = 0; row < (size_t)m; row++) {
@@ -1142,14 +1259,6 @@ void sum_mat
             B[col + row*ldb] += A[col + row*lda];
 }
 
-void transpose_mat(real_t *restrict A, size_t m, size_t n, real_t *restrict buffer_real_t)
-{
-    memcpy(buffer_real_t, A, m*n*sizeof(real_t));
-    for (size_t row = 0; row < m; row++)
-        for (size_t col = 0; col < n; col++)
-            A[row + col*m] = buffer_real_t[col + row*n];
-}
-
 void transpose_mat2(real_t *restrict A, size_t m, size_t n, real_t *restrict outp)
 {
     for (size_t row = 0; row < m; row++)
@@ -1379,27 +1488,6 @@ void coo_to_csr_and_csc
         free(cnt_bycol);
 }
 
-void row_means_csr(size_t indptr[], real_t *restrict values,
-                   real_t *restrict output, int_t m, int nthreads)
-{
-    int_t row = 0;
-    set_to_zero(values, m);
-    #pragma omp parallel for schedule(dynamic) num_threads(nthreads) \
-            shared(indptr, values, output, m)
-    for (row = 0; row < m; row++)
-    {
-        double rsum = 0;
-        for (size_t ix = indptr[row]; ix < indptr[row+(size_t)1]; ix++)
-            rsum += values[ix];
-        output[row] = rsum;
-    }
-    nthreads = cap_to_4(nthreads);
-    #pragma omp parallel for schedule(static) num_threads(nthreads) \
-            shared(indptr, output, m)
-    for (row = 0; row < m; row++)
-        output[row] /= (real_t)(indptr[row+(size_t)1] - indptr[row]);
-}
-
 bool should_stop_procedure = false;
 bool handle_is_locked = false;
 void set_interrup_global_variable(int_t s)
